const base = require('./webpack.config.base')
const merge = require('webpack-merge')
const MiniCssExtractPlugin = require('mini-css-extract-plugin')
const {CleanWebpackPlugin} = require('clean-webpack-plugin')
const OptimizeCssAssetsPlugin = require('optimize-css-assets-webpack-plugin')
const UglifyJsPlugin = require('uglifyjs-webpack-plugin')
const CopyWebpackPlugin = require('copy-webpack-plugin')

module.exports = merge(base, {
  output: {
    filename: 'static/js/[name].[contenthash:8].js',  //设置chunkhash可以使webpack根据两个chunk的内容决定是否改变hash; contenthash基于自身内容计算hash，不影响其它chunk
    chunkFilename: 'static/js/[name].[contenthash:8].js'  // 用于动态加载的chunk
  },

  optimization: { //设置代码的压缩处理
    moduleIds: 'hashed',  //webpack默认使用id自增的方式标志每个模块，一旦模块改变了导致引用的模块也改变，将其换成hash方式
    // namedChunks: true,
    runtimeChunk: true, //将包含chunks的映射关系的list单独从app.js抽离出来，因为每个chunk的id是基于内容hash出来的，所以每次改动app.js会影响缓存的命中
    minimizer: [
      new UglifyJsPlugin({
        // cache: true,
        parallel: true, //启用缓存并且启用多进程并行进行
        sourceMap: false, //不打包.map文件
        uglifyOptions: {
          warnings: false, //设置删除没有用处的代码时的警告
          compress: {
            loops: true,  //当do、while、for循环的条件可以确定时对其优化
            unused: true,  //干掉没有被引用的函数和变量，除非设置“keep_assign”，否则变量的简单赋值也不算被引用
            pure_funcs: ['console.log', 'console.info']  //干掉数组中的函数
          }
        },
        chunkFilter: (chunk) => {
          // `vendor` 模块不压缩
          if (chunk.name == 'vendors') {
            return false;
          }
          return true;
        }
      }),
      new OptimizeCssAssetsPlugin({
        assetNameRegExp: /\.css$/, //匹配需要压缩的css文件名
        cssProcessor: require('cssnano'), //压缩和优化css的处理器
        cssProcessorOptions: { safe: true, discardComments: { removeAll: true } },
        canPrint: true  //表示插件能够console打印出信息
      })
    ],
    splitChunks: {  // 自定义规则抽离想要的包
      chunks: 'all',
      cacheGroups: {  // 自定义配置决定哪些文件需要长期缓存
        // styles: {
        //   name: 'styles',
        //   test: /\.css$/,
        //   enforce: true
        // },
        vendors: { // 将一些不常变化的第三方库打包到vendor.js中
          test: /[\\/]node_modules[\\/]/,
          name: 'vendors',
          // enforce: true
        },
        commons: {  // 将多个页面同时引用的依赖打到一个包中，这里设置是2个
          test: /\.js$/,
          name: 'commons',
          minChunks: 2,
          reuseExistingChunk: true
        }
      }
    }
  },


  performance: {
    hints: "warning", // 枚举
    maxAssetSize: 30000000, // 整数类型（以字节为单位）
    maxEntrypointSize: 50000000, // 整数类型（以字节为单位）
    assetFilter: function(assetFilename) {
    // 提供资源文件名的断言函数
      return assetFilename.endsWith('.css') || assetFilename.endsWith('.js');
    }
  },

  module: {
    rules: [
      {
        test: /\.css$/,
        use: [
          {
            loader: MiniCssExtractPlugin.loader
          },
          'css-loader'
        ]
      }
    ]
  },

  plugins: [
    new MiniCssExtractPlugin({ //将css分离出来
      filename: 'static/css/[name].[contenthash:8].css' 
    }), 
    new CopyWebpackPlugin({
      patterns: [{
          from: './public'
      }]
    },{
      ignore: ['.*']
    }),
    new CleanWebpackPlugin()
  ]
})